/*********************************************
* This is the extension's main file. It runs when firefox loads, makes the first needed checks,
* and then loads the single engine.
* @Author: Eylon Yogev
*********************************************/

try {
    HeadupClass.prototype.LoadInits = function() {
        // inits IO path to extension folder
        gHeadup.InitIO();

        // loads the computer's platform to the Platrom variable
        gHeadup.GetPlatform();

        // init the computer Guid from the prefs or create a new one.
        gHeadup.InitComputerGuid();

        // send first load statistics has silverlight.
        gHeadup.SendFirstStat();
        
        // check that silverlight is installed and install if needed
        if (gHeadup.RunSilverlightInstallation()) {
            return false;
        }

        return true;
    }

    // when a page DOM content is loaded (befoer other stuff like images and so)
    HeadupClass.prototype.OnDOMLoadedEvent = function(event, win) {
        var winID = win.Guid;
        var dom = event.originalTarget;



        var tab = gHeadup.GetContainingTab(dom, win);
        dom.Tab = tab;
        dom.domLoadedRaised = true;

        gHeadup.OnSomeDomLoaded(dom, winID);
    }

    // raised on each page change. Used to raise dom loaded when user clicks back/forward.
    HeadupClass.prototype.OnPageShow = function(event, win) {
        var dom = event.originalTarget;
        if (dom.domLoadedRaised == null || !dom.domLoadedRaised) {
            gHeadup.OnDOMLoadedEvent(event, win);
        }
    }    

    //function OnFrameLoaded(event, win) {
    //    var nativeWrapper = event.originalTarget;
    //    var doc = nativeWrapper.contentDocument;
    //    var tab = null;

    //    if (nativeWrapper.id != null && nativeWrapper.id == 'canvas_frame') {
    //        if (nativeWrapper.ownerDocument != null) {
    //            tab = GetContainingTab(nativeWrapper.ownerDocument, win);
    //            doc.Tab = tab;
    //            //OnSomeDomLoaded(doc);
    //        }
    //    }
    //}


    HeadupClass.prototype.OnSomeDomLoaded = function(page, winID) {
        try {
            if (gHeadup.isExtensionEnabled) {

                var tab = page.Tab;
                if (tab == null) {
                    // alert('tab is null!');
                    return;
                }
                gHeadup.InitPage(page);
                gHeadup.FirePageLoadedEvent(page, winID);
            }
        }
        catch (err) {
            gHeadup.LogError('OnSomeDomLoaded', err);
        }
    }

    HeadupClass.prototype.InitPage = function(page) {
        try {
            page.Guid = gHeadup.NextGuid();
            if (page.CharmGuids == undefined) {
                page.CharmGuids = {};
            }
            page.Tab.Doms[page.Guid] = page;
        }
        catch (err) {
            gHeadup.LogError('InitPage', err);
        }
    }

    HeadupClass.prototype.OnPageUnload = function(aEvent, window) {
        if (aEvent.originalTarget instanceof HTMLDocument) {
            var doc = aEvent.originalTarget;
            doc.domLoadedRaised = false;

            var tab = doc.Tab;
            if (tab != null)
                gHeadup.PageUnload(tab);
        }
    }

    HeadupClass.prototype.PageUnload = function(tab) {
        try {
            if (gHeadup.SingleEngineWindow == null) return;

            tab.Doms = {};
            var data = {};
            data['tabid'] = tab.Guid;

            gHeadup.FireEventInSingleWindow("PageUnloaded", data);
        }
        catch (err) {
            gHeadup.LogError('PageUnload', err);
        }
    }

    HeadupClass.prototype.FirstInit = function() {
        try {
            // check that silverlight is installed. be checked this before but there is still a change that
            // we can get here. if the installed failed for example.
            var checkSuccess = this.FirstInstallationCheck();
            if (checkSuccess == false)
                return;

            this.RegisterObserver();

            window.addEventListener("close", gHeadup.OnHeadupClose, true, true);

            // Event when fire fox is closed
            window.addEventListener("close", gHeadup.OnApplicationExit, true, true);
            window.addEventListener("focus", gHeadup.OnSingleWindowFocus, true, true);
            window.addEventListener("blur", gHeadup.OnSingleWindowBlur, true, true);
        }
        catch (err) {
            gHeadup.LogError('FirstInit', err);
        }
    }

    HeadupClass.prototype.FirstInstallationCheck = function() {

        gHeadup.isSilverlightInstalled = gHeadup.IsSilverlightInstalled();
        var firstLoaded = gHeadup.GetStringPref('FirstLoaded2');
        gHeadup.SetStringPref('FirstLoaded2', 'true');
        if (gHeadup.isSilverlightInstalled == false) {
            if (firstLoaded == 'true') {
                gHeadup.SetDisabled(true);
            }
            else {
                // send statistic first load with no silverlight
                this.OpenNoPluginPage();
            }

            return false;
        }
        else if (firstLoaded != 'true') {
            // first time
        }

        return true;
    }

    HeadupClass.prototype.OpenNoPluginPage = function() {
        setTimeout(function() {
            gHeadup.OpenUrlInInstalltaionTab(gHeadup.Consts.NoPluginPage);
        }, 1000);
    }

    HeadupClass.prototype.OpenStartPage = function() {
        setTimeout(function() {
            gHeadup.OpenUrlInInstalltaionTab(gHeadup.Consts.LoadingURL);
        }, 1000);
    }

    HeadupClass.prototype.RegisterObserver = function() {
        httpRequestObserver.register();
    }


    HeadupClass.prototype.OnHeadupClose = function(event) {
        try {

            event.preventDefault();
            var tabid = gHeadup.GetActiveTabGuid();
            gHeadup.HideSingleWindow(tabid);
            var data = {};
            data['tabid'] = tabid;
            gHeadup.FireEventInSingleWindow("WindowClosed", data);
        }
        catch (err) {
            gHeadup.LogError('OnHeadupClose', err);
        }
    }

    HeadupClass.prototype.OnApplicationExit = function(closedWindow) {
        try {
            var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
            var enumerator = wm.getEnumerator(null);
            while (enumerator.hasMoreElements()) {
                try {
                    var win = enumerator.getNext();
                    if (win.gBrowser != null && win.location != gHeadup.Consts.SingleEngineChromPath && win != closedWindow) {
                        return;
                    }
                }
                catch (err) {
                    gHeadup.LogError('OnApplicationExit - while loop', err);
                }
            }
            gHeadup.ExitApplication();
        }
        catch (err) {
            gHeadup.LogError('OnApplicationExit', err);
        }
    }

    HeadupClass.prototype.ExitApplication = function() {
        try {
            gHeadup.CloseAllXulWindows();
            gHeadup.InternalHideSingleWindow();
            gHeadup.FireEventInSingleWindow("FirefoxExit", null);
            gHeadup.SingleEngineWindow.content.src = 'about:blank';
        }
        catch (err) {
            gHeadup.LogError('ExitApplication', err);
        }

        try {
            window.close();
        }
        catch (err) {
        }
    }
}
catch (err) {
    gHeadup.LogError('error prototyping FirefoxEvents: ' + err);
}