/*********************************************
* This file in resposible on any IO in the extension.
* uses the nsIFile class in firefox.
* For more info: https://developer.mozilla.org/en/NsIFile
* @Author: Eylon Yogev
*********************************************/

        // holds the path to the extension directory.
        // Usuallys: %userprofile%\Application Data\Mozilla\Firefox\Profiles\[Filefox Profile]\extensions\headup@semantinet.com
        HeadupClass.prototype.extensionPath = null;

        // holds the path to the firefox profile folder
        HeadupClass.prototype.profilePath = null;

        // holds the path to our datra folder. This is a subdirectory named 'data' in the extension folder.
        HeadupClass.prototype.userDataPath = null;

        HeadupClass.prototype.inited = false;
        // inits pathes for future use
        HeadupClass.prototype.InitIO = function() {
            if (this.inited)
                return;

            this.GetExtensionDirectory();
            this.GetProfileDirectory();
            this.userDataPath = this.extensionPath + 'data';
            this.inited = true;
        }

        HeadupClass.prototype.ReadBinaryFileAsync = function(params) {
            try {
                var filePath = params["filepath"];
                var opid = params['opId'];
                var file = this.FilePathToObject(filePath);
                var ios = Components.classes["@mozilla.org/network/io-service;1"]
                  .getService(Components.interfaces.nsIIOService);
                var fileURI = ios.newFileURI(file);
                var channel = ios.newChannelFromURI(fileURI);
                var observer = {
                    onStreamComplete: function(aLoader, aContext, aStatus, aLength, aResult) {
                        var data = {};
                        var unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
                        unicodeConverter.charset = "ASCII";
                        var body = unicodeConverter.convertFromByteArray(aResult, aLength);
                        data['responsetext'] = body;
                        data['opid'] = opid;
                        gHeadup.FireEventInSingleWindow('MethodCallback', data);
                    }
                };
                var sl = Components.classes["@mozilla.org/network/stream-loader;1"].
                   createInstance(Components.interfaces.nsIStreamLoader);

                sl.init(observer);
                channel.asyncOpen(sl, channel);
            }

            catch (err) {
                gHeadup.LogError('ReadBinaryFileAsync', err);
            }
        }
        
        // gets the bytes of a binary file.
        // file - a nsILocalFile to read from.
        HeadupClass.prototype.GetBinaryFileData = function(file) {
            var istream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                        .createInstance(Components.interfaces.nsIFileInputStream);
            istream.init(file, -1, -1, false);
            var bstream = Components.classes["@mozilla.org/binaryinputstream;1"]
                        .createInstance(Components.interfaces.nsIBinaryInputStream);
            bstream.setInputStream(istream);
            var bytes = bstream.readBytes(bstream.available());
            bstream.close();
            istream.close();
            return bytes;
        }

        // gets the bytes of a binary file.
        // path - the path of the file to read from.
        HeadupClass.prototype.GetBinaryFileDataByPath = function(path) {

            var file = this.FilePathToObject(path);
            return this.GetBinaryFileData(file);
        }

        // returns whether  the file path exists or not.
        HeadupClass.prototype.FileExists = function(path) {
            try {
                this.InitIO();
                var file = this.FilePathToObject(path);
                return file.exists();
            }
            catch (err) {
                return false;
            }
        }

        // returns whether a headup file path exists or not.
        HeadupClass.prototype.HeadupFileExists = function(path) {
            try {
                this.InitIO();
                var file = this.FileSpecialPathToObject(path);
                return file.exists();
            }
            catch (err) {
                return false;
            }
        }

        // Gets an array of file names in a given directory
        HeadupClass.prototype.GetFiles = function(dirPath) {
            try {
                var dir = this.FilePathToObject(dirPath);
                return gHeadup.GetAnyFiles(dir);
            }
            catch (err) {
            }
        }

        // Gets an array of file names in a given directory
        HeadupClass.prototype.GetSpecialFiles = function(dirPath) {
            try {
                var dir = gHeadup.FileSpecialPathToObject(dirPath);
                return gHeadup.GetAnyFiles(dir);
            }
            catch (err) {
            }
        }

        HeadupClass.prototype.GetAnyFiles = function(dir) {
            var entries = dir.directoryEntries;
            var files = [];
            var i = 0;
            while (entries.hasMoreElements()) {
                var entry = entries.getNext();
                entry.QueryInterface(Components.interfaces.nsIFile);
                if (entry.isFile()) {
                    files[i] = entry.leafName;
                }
                i++;
            }
            return files;
        }

        // Searches for a file recursivly under a specific directory.
        // if many found, returns the last modified.
        // dir - the directory to start with
        // filename - the file to search for
        HeadupClass.prototype.GetLatestFileFromIS = function(dir, filename) {
            var files = [];
            this.FindFile(dir, filename, files);
            var latest = this.GetLastModifiedFile(files);
            return latest;
        }

        // returns the last modifier parent folder from the given list.
        // list - a list of files.
        HeadupClass.prototype.GetLastModifiedFile = function(list) {
            var latestFile = null;
            var maxTicks = 0;
            for (var i = 0; i < list.length; i++) {
                if (list[i].parent.lastModifiedTime > maxTicks) {
                    maxTicks = list[i];
                    latestFile = list[i];
                }
            }

            return latestFile;
        }

        // recursivly finds a file in a given directory. adds all intances found to the list.
        // dir - the directory to search in.
        // fileName - the filename to search for.
        // list - the list the results are added to.
        HeadupClass.prototype.FindFile = function(dir, fileName, list) {
            // first looks for files right below the given directory.
            var entries = dir.directoryEntries;
            while (entries.hasMoreElements()) {
                var entry = entries.getNext();
                entry.QueryInterface(Components.interfaces.nsIFile);
                if (entry.isFile()) {
                    if (entry.leafName.toLowerCase() == fileName.toLowerCase()) {
                        list[list.length] = entry;
                    }
                }
            }

            // now dig deeper recursivly in the sub directories.
            var entries = dir.directoryEntries;
            while (entries.hasMoreElements()) {
                var entry = entries.getNext();
                entry.QueryInterface(Components.interfaces.nsIFile);
                if (entry.isDirectory()) {
                    this.FindFile(entry, fileName, list);
                }
            }
        }

        // writes a text file to the hard disk.
        // fileName - the full path of the file to write. if subdirectories are missing if the full path, it will create them.
        // data - the data to write.
        HeadupClass.prototype.WriteTextFile = function(fileName, data) {
            try {
                this.DeleteFile(fileName, false);
                var aFile = this.FilePathToObject(fileName);
                return gHeadup.WriteText(aFile, data);
            }
            catch (err) {
                gHeadup.LogError('WriteTextFile', err);
                return false;
            }
        }

        // writes a text file to the hard disk.
        // fileName - the full path of the file to write. if subdirectories are missing if the full path, it will create them.
        // data - the data to write.
        HeadupClass.prototype.WriteSpecialTextFile = function(fileName, data) {
            try {
                this.DeleteSpecialFile(fileName, false);
                var aFile = this.FileSpecialPathToObject(fileName);
                return gHeadup.WriteText(aFile, data);
            }
            catch (err) {
                gHeadup.LogError('WriteTextFile', err);
                return false;
            }
        }

        HeadupClass.prototype.WriteText = function(aFile, data) {
            try {
                aFile.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 600);
                // file is nsIFile, data is a string
                var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
                         .createInstance(Components.interfaces.nsIFileOutputStream);

                // use 0x02 | 0x10 to open file for appending.
                foStream.init(aFile, 0x02 | 0x08 | 0x20, 0666, 0);
                // write, create, truncate
                // In a c file operation, we have no need to set file mode with or operation,
                // directly using "r" or "w" usually.
                foStream.write(data, data.length);
                // close stream.
                foStream.close();

                return true;
            }
            catch (err) {
                gHeadup.LogError('WriteText', err);
                return false;
            }
        }

        // writes a binary file to the hard disk. will create all subdirectories needed in the way.
        // fileName - the full path file name.
        // data - the binary data to write.
        HeadupClass.prototype.WriteBinaryFile = function (fileName, data) {
            try {
                this.DeleteFile(fileName, false);
                var aFile = this.FilePathToObject(fileName);
                aFile.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 600);

                var fileStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
                fileStream.init(aFile, 0x02 | 0x20, 0600, 0); // PR_WRONLY | PR_TRUNCATE

                var stream = Components.classes["@mozilla.org/binaryoutputstream;1"].createInstance(Components.interfaces.nsIBinaryOutputStream);
                stream.setOutputStream(fileStream);

                stream.write(data, data.length);

                stream.close();
                fileStream.close();

                return true;
            }
            catch (err) {
                gHeadup.LogError('WriteBinaryFile', err);
                return false;
            }
       }

        // reads a text file from the hard disk.
        // fileName - the full path file to read from.
       HeadupClass.prototype.ReadTextFile = function(fileName) {
           try {
               gHeadup.InitIO();
               var file = gHeadup.FilePathToObject(fileName);
               return gHeadup.GetTextData(file);
           }
           catch (err) {
               gHeadup.LogError('ReadTextFile', err);
           }
       }

        // internal function. gets the textual data of a given file object.
       HeadupClass.prototype.GetTextData = function(file) {
           try {
               var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"]
                            .createInstance(Components.interfaces.nsIFileInputStream);
               fstream.init(file, -1, 0, 0);

               var charset = /* Need to find out what the character encoding is. Using UTF-8 for this example: */"UTF-8";
               var replacementChar = Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER;
               var is = Components.classes["@mozilla.org/intl/converter-input-stream;1"]
                           .createInstance(Components.interfaces.nsIConverterInputStream);
               is.init(fstream, charset, 1024, replacementChar);

               var data = new StringBuffer();
               var str = {};
               while (is.readString(4096, str) != 0) {
                   data.append(str.value);
               }
               // close streams.
               fstream.close();
               is.close();
               return data.toString();

           }
           catch (err) {
               gHeadup.LogError('ReadTextFile', err);
           }
       }

        // gets the path of the extension. Meaning, the path where the install.rdf files is.
        HeadupClass.prototype.GetExtensionDirectory = function() {
            if (this.extensionPath == null) {
                // the extension's id from install.rdf
                var MY_ID = "headup@semantinet.com";
                var em = Components.classes["@mozilla.org/extensions/manager;1"].
         getService(Components.interfaces.nsIExtensionManager);
                // the path may use forward slash ("/") as the delimiter
                var file = em.getInstallLocation(MY_ID).getItemFile(MY_ID, "install.rdf");
                this.extensionPath = file.path.toString().replace('install.rdf', '');
            }

            return this.extensionPath;
        }

        // get the current firefox directory.
        // for example: C:\Documents and Settings\[username]\Application Data\Mozilla\Firefox\Profiles\l1qcgb4y.Dev
        HeadupClass.prototype.GetProfileDirectory = function() {
            if (this.profilePath == null) {
                // get profile directory
                var file = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("ProfD", Components.interfaces.nsIFile);

                // NOTE: "file" is an object that implements nsIFile. If you want the
                // file system path, use file.path
                file.append('headup');
                this.profilePath = file.path.toString();
            }

            return this.profilePath;
        }

        // reads a binary stored head file from the data folder
        HeadupClass.prototype.ReadSpecialFile = function(filename) {
            try {
                this.InitIO();
                var file = this.FileSpecialPathToObject(filename);
                return this.GetBinaryFileData(file);
            }
            catch (err) {
                gHeadup.LogError('ReadSpecialFile: ' + filename, err);
            }
        }

        // reads a textual head file form the data folder.
        HeadupClass.prototype.ReadSpecialTextFile = function(filename) {
            try {
                var file = this.FileSpecialPathToObject(filename);
                return this.GetTextData(file);
            }
            catch (err) {
                gHeadup.LogError('ReadSpecialFile: ' + filename, err);
            }
        }

        // reads a binary file from the disk.
        // filename - the full path file to read from.
        HeadupClass.prototype.ReadBinaryFile = function(filename) {
            try {
                var file = this.FilePathToObject(filename);
                if (file.exists())
                    return this.GetBinaryFileData(file);
                else
                    return null;
            }
            catch (err) {
                gHeadup.LogError('ReadBinaryFile: ' + filename, err);
            }
        }

        // deletes a file or a directory from a disk.
        // fileName - the full path file or directory to delete.
        // recursive - if true, deletes all subdirectories too. relevant only for folder.
        HeadupClass.prototype.DeleteFile = function(fileName, recursive) {
            try {
                var file = this.FilePathToObject(fileName);
                if (file.exists()) {
                    file.remove(recursive);
                    return true;
                }
                return false;
            }
            catch (err) {
                gHeadup.LogError('DeleteFile', err);
                return false;
            }
        }

        // deletes a file or a directory from a disk.
        // fileName - the full path file or directory to delete.
        // recursive - if true, deletes all subdirectories too. relevant only for folder.
        HeadupClass.prototype.DeleteSpecialFile = function(fileName, recursive) {
            try {
                var file = gHeadup.FileSpecialPathToObject(fileName);
                if (file.exists()) {
                    file.remove(recursive);
                    return true;
                }
                return false;
            }
            catch (err) {
                gHeadup.LogError('DeleteSpecialFile', err);
                return false;
            }
        }

        // returns a file object by the fileName in the uesr's folder
        HeadupClass.prototype.FilePathToObject = function (fileName) {
            var file = Components.classes["@mozilla.org/file/local;1"]
                             .createInstance(Components.interfaces.nsILocalFile);
                file.initWithPath(this.profilePath);
                var folders = fileName.split('\\');
                for (var i = 0; i < folders.length; i++) {
                    file.append(folders[i]);
                }
                return file;
            }

        // converts a file or directory path to a file object. creates all subdirectories needed in the way.
            HeadupClass.prototype.FileSpecialPathToObject = function(fileName) {
                var file = Components.classes["@mozilla.org/file/local;1"]
                     .createInstance(Components.interfaces.nsILocalFile);
                file.initWithPath(this.userDataPath);
                var folders = fileName.split('\\');
                for (var i = 0; i < folders.length; i++) {
                    file.append(folders[i]);
                }
                return file;
            }

        // returns the file size.
        // fileName - the pull path file name.
        HeadupClass.prototype.FileSize = function(fileName) {
            var file = this.FilePathToObject(fileName);
            return file.fileSize;
        }

        // returns the size of the given directory. This includes all subfolders as well.
        // dirPath - the fill directory path.
        HeadupClass.prototype.DirectorySize = function(dirPath) {
            try {
                var dir = this.FilePathToObject(dirPath);
                return this.DirectorySizeRec(dir);
            }
            catch (err) {
                gHeadup.LogError('DirectorySize', err);
            }
        }

        // for inner use. recursivly calculates the folder size.
        // dir - the directory object to calculate.
        HeadupClass.prototype.DirectorySizeRec = function(dir) {
            var sum = 0;
            var entries = dir.directoryEntries;
            while (entries.hasMoreElements()) {
                var entry = entries.getNext();
                entry.QueryInterface(Components.interfaces.nsIFile);
                if (entry.isFile()) {
                    sum += entry.fileSize;
                }
                else {
                    sum += this.DirectorySizeRec(entry);
                }
            }
            return sum;
        }

        // moves a file to a new location. Used mainly for renaming.
        HeadupClass.prototype.MoveFile = function(fileName, dir, newName) {
            try {
                var file = this.FilePathToObject(fileName);
                var folder = this.FilePathToObject(dir);
                file.moveTo(folder, newName);
                return true;
            }
            catch (err) {
                gHeadup.LogError('MoveFile', err);
                return false;
            }
        }
