/*********************************
* WindowLoad.js
* runs with each open of a window. In the first time run, it loads the single engine.
* for the others, it just registers to the existing single engine using the RegisterNewWindow method.
* It also register to all the windows events, and notifies the single engine of each event.
* @Author: Eylon Yogev. 2008.
*********************************/


try {

    // a referece to the mighty singel window.
    HeadupClass.prototype.singleWindow;

    // maintains a bool indicating weather this window is focused or not.
    HeadupClass.prototype.isFocused = false;

    // indicates if this window has been inited. The single engine changes this to true.
    HeadupClass.prototype.inited = false;

    // holds wether the extension is enabled
    HeadupClass.prototype.isExtensionEnabled = false;


    HeadupClass.prototype.AttachWindowEvents = function() {
        // register to the load event.
        window.addEventListener("load", gHeadup.OnWindowLoad, false);

        // register to the close event.
        window.addEventListener("close", gHeadup.OnWindowUnload, true, true);

        window.addEventListener("mousedown", gHeadup.OnMouseDown, true, true);
        window.addEventListener("mouseup", gHeadup.OnMouseUp, true, true);

    }

    HeadupClass.prototype.isMouseDown = false;

    HeadupClass.prototype.OnMouseDown = function() {
        this.isMouseDown = true;
    }

    HeadupClass.prototype.OnMouseUp = function() {
        this.isMouseDown = false;
    }


    // main load method. checks for exisiting single engine. If so, register to it. If not
    // load the single engine.
    HeadupClass.prototype.OnWindowLoad = function() {
        try {

            // check if the extension is enabled
            gHeadup.isExtensionEnabled = true;  //gHeadup.GetStringPref(gHeadup.Consts.EnabledPref) == 'true';
            // set the statusbar icon
            gHeadup.SetEnableDisableIcon(gHeadup.isExtensionEnabled);
            // if not enabled - return
//            if (gHeadup.isExtensionEnabled == false) {
//                // inits the window's statusbar
//                gHeadup.InitContextMenu(window);
//                // enable disable toggle icon
//                gHeadup.ChangeMenuItemProperty("charm-menuitem-Enable", "disabled", "false");
//                return;
//            }
            // find the single window
            gHeadup.singleWindow = gHeadup.FindSingleWindow();
            // exists?
            if (gHeadup.singleWindow == null) {
                // this is the first window loading. load the single engine.
                gHeadup.LoadSingleEngine();
            }
            else {
                // a single engine is already loaded. register to it.
                gHeadup.singleWindow.gHeadup.RegisterNewWindow(window, false);
            }
        }
        catch (err) {
            gHeadup.LogError('OnWindowLoad', err);
        }
    }

    HeadupClass.prototype.ChangeMenuItemProperty = function(itemid, name, value) {        
        var element = document.getElementById(itemid);
        if (element != null)
            element.setAttribute(name, value);     
    }

    HeadupClass.prototype.OnWindowUnload = function(aEvent) {
        try {
            if (gHeadup.singleWindow != null) {
                try {
                    var num = gBrowser.browsers.length;
                    for (var i = 0; i < num; i++) {
                        var tab = gBrowser.getBrowserAtIndex(i);
                        gHeadup.singleWindow.gHeadup.PageUnload(tab);
                    }
                    gHeadup.DetachEvents();
                }
                catch (err) {
                }

                // tell the single window that we are closed to he can decide if we are the last window and close himself.
                gHeadup.singleWindow.gHeadup.OnApplicationExit(window);
                gHeadup.singleWindow = new HeadupClass();
            }
        }
        catch (err) {
            gHeadup.LogError('OnWindowUnload', err);
        }
    }

    // handle Ubiquity query request. send the text from the ubiquity command
    // to the single engine
    HeadupClass.prototype.HeadupUbiquityQuery = function(text) {
        try {
            if (this.isExtensionEnabled) {
                var data = {};
                data['text'] = text;
                data['tabid'] = gBrowser.selectedBrowser.Guid;
                this.singleWindow.gHeadup.FireEventInSingleWindow("UbiquityQuery", data);
            }
        }
        catch (err) {
            gHeadup.LogError('HeadupUbiquityQuery', err);
        }
    }

    // global function to call the method in gHeadup
    function HeadupUbiquityQuery(text) {
        gHeadup.HeadupUbiquityQuery(text);
    }

    // check to see if this is the first instnce of firefox opened, return true if it is, otherwise false.
    HeadupClass.prototype.FindSingleWindow = function() {
        try {
            var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
            var enumerator = wm.getEnumerator(null);
            while (enumerator.hasMoreElements()) {
                var win = enumerator.getNext();

                if (win.location == gHeadup.Consts.SingleEngineChromPath) {
                    return win;
                }
            }

            return null;
        }
        catch (err) {
            gHeadup.LogError('FindSingleWindow', err);
        }
    }

    // sets the statusbar icon to enabled or disabled
    HeadupClass.prototype.SetEnableDisableIcon = function(trueOrFalse) {
        var iconElement = document.getElementById(gHeadup.Consts.KeybitzEnableIconId);
        if (trueOrFalse == true) {
            iconElement.setAttribute('src', gHeadup.Consts.EnabledIconPath);
            //document.getElementById('charm-menuitem-Enable').label = gHeadup.Consts.DisableText;
        }
        else {
            iconElement.setAttribute('src', gHeadup.Consts.DisabledIconPath);
            //document.getElementById('charm-menuitem-Enable').label = gHeadup.Consts.EnableText;
        }
    }

    // loads the holly single window.
    HeadupClass.prototype.LoadSingleEngine = function() {
        try {
            var features = null;
            if (gHeadup.Platform == 'mac') {
                // mac features: (see the popup=no)
                features = 'titlebar=no,popup=no,resizable=no,scrollbars=no';
            }
            else {

                // windows features: (see the popup=yes
                features = 'titlebar=no,popup=yes,resizable=no,scrollbars=no';
            }
            this.singleWindow = window.openDialog(gHeadup.Consts.SingleEngineChromPath, '', features, null);
        }
        catch (err) {
            gHeadup.LogError('LoadSingleEngine', err);
        }
    }

    // register to all events we need from the window
    HeadupClass.prototype.RegisterToEvents = function() {
        // register to focus events.
        window.addEventListener("focus", gHeadup.OnWindowFocus, true, true);
        window.addEventListener("blur", gHeadup.OnWindowBlur, true, true);

        // register to all tabs events.
        var container = gBrowser.tabContainer;
        container.addEventListener("TabSelect", gHeadup.TabSelect, false);
        container.addEventListener("TabOpen", gHeadup.TabOpen, false);
        container.addEventListener("TabMove", gHeadup.TabMove, false);
        container.addEventListener("TabClose", gHeadup.TabClose, false);

        // Register to a new page load event.
        window.addEventListener("DOMContentLoaded", gHeadup.OnDOMLoadedEvent, false);
        window.addEventListener("pageshow", gHeadup.OnPageShow, false);        
        window.addEventListener("pagehide", gHeadup.OnPageUnload, false);
        //window.addEventListener("DOMFrameContentLoaded", OnFrameLoaded, false);
    }

    // detaches all events registered. This is used for disabling the application.
    HeadupClass.prototype.DetachEvents = function() {
        window.removeEventListener("focus", gHeadup.OnWindowFocus, true, true);
        window.removeEventListener("blur", gHeadup.OnWindowBlur, true, true);

        // register to all tabs events.
        var container = gBrowser.tabContainer;
        container.removeEventListener("TabSelect", gHeadup.TabSelect, false);
        container.removeEventListener("TabOpen", gHeadup.TabOpen, false);
        container.removeEventListener("TabMove", gHeadup.TabMove, false);
        container.removeEventListener("TabClose", gHeadup.TabClose, false);

        // Register to a new page load event.
        window.removeEventListener("DOMContentLoaded", gHeadup.OnDOMLoadedEvent, false);
        window.removeEventListener("pageshow", gHeadup.OnPageShow, false);
        window.removeEventListener("pagehide", gHeadup.OnPageUnload, false);

        gHeadup.isFocused = false;
        gHeadup.inited = false;
        gHeadup.isExtensionEnabled = false;
        gHeadup.isMouseDown = false;
    }

    // the page has been unloaded
    HeadupClass.prototype.OnPageUnload = function(aEvent) {
        gHeadup.singleWindow.gHeadup.OnPageUnload(aEvent, window);
    }

    // a new page has been loaded event
    HeadupClass.prototype.OnDOMLoadedEvent = function(event) {
        gHeadup.singleWindow.gHeadup.OnDOMLoadedEvent(event, window);
    }

    // a new page has been loaded event
    HeadupClass.prototype.OnPageShow = function(event) {
        gHeadup.singleWindow.gHeadup.OnPageShow(event, window);
    }

    HeadupClass.prototype.OnFrameLoaded = function(event) {
        gHeadup.singleWindow.gHeadup.OnFrameLoaded(event, window);
    }

    // this window has been focused
    HeadupClass.prototype.OnWindowFocus = function(event) {
        if (gHeadup.isFocused == false) {
            gHeadup.isFocused = true;
            gHeadup.singleWindow.gHeadup.OnWindowFocus(window);
        }
    }

    // this window has been blured
    HeadupClass.prototype.OnWindowBlur = function(event) {
        if (gHeadup.isFocused == true) {
            gHeadup.isFocused = false;
            gHeadup.singleWindow.gHeadup.OnWindowBlur(window);
        }
    }

    // a tab in this window has been selected
    HeadupClass.prototype.TabSelect = function(event) {
        gHeadup.singleWindow.gHeadup.TabSelect(event, window);
    }

    // a new tab in this window has been opened
    HeadupClass.prototype.TabOpen = function(event) {
        gHeadup.singleWindow.gHeadup.TabOpen(event, window);
    }

    // a tab has been moved. we don't use this now, but still good to have in mind
    HeadupClass.prototype.TabMove = function(event, win) {
        gHeadup.singleWindow.gHeadup.TabMove(event, window);
    }

    // a tab has been closed.
    HeadupClass.prototype.TabClose = function(event) {
        gHeadup.singleWindow.gHeadup.TabClose(event, window);
    }


    HeadupClass.prototype.ChangeHeadupStatus = function(event) {
        if (this.singleWindow != null && this.singleWindow.gHeadup.isClosed == false) {
            this.singleWindow.gHeadup.ChangeHeadupStatus(event);
        }
        else if (gHeadup.IsSilverlightInstalled()) {
            this.LoadSingleEngine();
            gHeadup.SetEnableDisableIcon(true);
        }
        else {
            gHeadup.SetStringPref(gHeadup.Consts.EnabledPref, 'true');
            gBrowser.selectedTab = gBrowser.addTab(gHeadup.Consts.SilverlightMissing);
        }
    }

    HeadupClass.prototype.OnMouseOver = function(event) {
        window.setCursor('pointer');
    }

    HeadupClass.prototype.OnMouseOut = function(event) {
        window.setCursor('default');
    }

    HeadupClass.prototype.InitContextMenu = function(win) {
        try {
            var charmMenu = win.document.getElementById('charm-main-menupopup');
            var bringToFrontItem = win.document.getElementById('charm-menuitem-BringToFront');
            var showDebugItem = win.document.getElementById('charm-menuitem-ShowDebugScreen');

            if (gHeadup.Platform == 'mac') {

                // show the bring to front menu item
                bringToFrontItem.hidden = false;
            }
            else {
                bringToFrontItem.hidden = true;
            }

            var dbg = (gHeadup.GetStringPref(gHeadup.Consts.DebugPref) == 'true')
            if (dbg) {
                showDebugItem.hidden = false;
            }
            else {
                showDebugItem.hidden = true;
            }
        }
        catch (err) {
            //gHeadup.LogError('InitContextMenu', err);
        }
    }


    // only running code in main context. attach to window event.
    gHeadup.AttachWindowEvents();
}
catch (err) {
    LogError('error prototyping WindowLoad: ' + err);
}