try {

    HeadupClass.prototype.GetSelectionData = function(tabid, docid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            if (dom == null)
                return null;
            var selection = gHeadup.GetSelection(dom.defaultView);
            if (selection == null || selection.rangeCount == 0){
                return null;
            }
            

            var range = selection.getRangeAt(0);

            var result = {};
            result['selectedText'] = selection.toString();
            result['elementText'] = range.commonAncestorContainer.textContent;
            result['offset'] = range.startOffset;
            return result;
        }
        catch (err) {           
            gHeadup.LogError('GetSelectedData', err);
            return null;
        }
    }

    HeadupClass.prototype.GetDocumentElement = function(tabid, docid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var guid = gHeadup.GetElementGuid(dom.documentElement, dom);
            return guid;
        }
        catch (err) {
            gHeadup.LogError('GetBody', err);
            return null;
        }
    }


    HeadupClass.prototype.GetTitle = function(tabid, docid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            return dom.title;
        }
        catch (err) {
            gHeadup.LogError('GetTitle', err);
            return null;
        }
    }

    HeadupClass.prototype.GetLocation = function(tabid, docid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            if (dom == null)
                return null;
            return dom.location.href;
        }
        catch (err) {
            return null;
        }
    }

    HeadupClass.prototype.GetLocationByTabId = function(id) {
        try {
            var dom = gHeadup.GetTabDomByTabId(id);
            if (dom == null)
                return null;
            return dom.location.href;
        }
        catch (err) {
            gHeadup.LogError('GetLocationByTabId', err);
            return null;
        }
    }

    HeadupClass.prototype.GetElementsByName = function(tabid, docid, name) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var elements = dom.getElementsByName(name);
            return this.ElementsToDataObject(elements, dom);
        }
        catch (err) {
            gHeadup.LogError('GetElementsByName', err);
        }
    }

    HeadupClass.prototype.GetElementsByTag = function(tabid, docid, name) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var elements = dom.getElementsByTagName(name);
            return this.ElementsToDataObject(elements, dom);
        }
        catch (err) {
            gHeadup.LogError('GetElementsByTag', err);
        }
    }

    HeadupClass.prototype.GetElementsByTagWithProperties = function(tabid, docid, name, properties, limit) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var elements = dom.getElementsByTagName(name);
            var guidsList = gHeadup.ElementsToDataObject(elements, dom);
            var length = Math.min(guidsList.length, limit);
            var result = new Array(length);
            
            for (var i = 0; i < length; i++) {
                result[i] = new Array(2);
                result[i][0] = guidsList[i];
                result[i][1] = gHeadup.GetPropertiesFromElement(elements[i], properties);
            }
            return result;
        }
        catch (err) {
            gHeadup.LogError('GetElementsByTagWithProperties', err);
        }
    }

    HeadupClass.prototype.ElementsToDataObject = function(elements, dom) {
        var data = new Array(elements.length);
        for (var i = 0; i < elements.length; i++) {
            var currnet = elements[i];
            var guid = gHeadup.GetElementGuid(currnet, dom);
            data[i] = guid;
        }
        return data;
    }

    HeadupClass.prototype.GetBody = function(tabid, docid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var guid = gHeadup.GetElementGuid(dom.body, dom);
            return guid;
        }
        catch (err) {
            gHeadup.LogError('GetBody', err);
            return null;
        }
    }

    HeadupClass.prototype.CreateElement = function(tabid, docid, name) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var element = dom.createElement(name);
            var guid = gHeadup.GetElementGuid(element, dom);
            return guid;
        }
        catch (err) {
            gHeadup.LogError('CreateElement', err);
        }
    }

    HeadupClass.prototype.EvalOnTab = function(tabid, docid, text) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            gHeadup.EvalOnPage(dom, text);
        }
        catch (err) {
            gHeadup.LogError('EvalOnTab', err);
        }
    }

    HeadupClass.prototype.GetElementById = function(tabid, docid, id) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var element = dom.getElementById(id);
            if (element == null) {
                return null;
            }
            var guid = gHeadup.GetElementGuid(element, dom);
            return guid;
        }
        catch (err) {
            gHeadup.LogError('GetElementById', err);
            return null;
        }
    }

    HeadupClass.prototype.Evaluate = function(tabid, docid, xPath) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var textNodes = dom.evaluate(xPath, dom, null, XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE, null);
            var data = new Array(textNodes.snapshotLength);
            for (var i = 0; i < textNodes.snapshotLength; i++) {
                var textNode = textNodes.snapshotItem(i);
                data[i] = gHeadup.GetElementGuid(textNode, dom);
                //data[i] = new Array();
                //data[i][0] = gHeadup.GetElementGuid(textNode, dom);

            }
            return data;
        }
        catch (err) {
            gHeadup.LogError('Evaluate', err);
            return null;
        }
    }

    

    HeadupClass.prototype.IsEnabled = function(tabid, docid) {
        try {
            var tab = gHeadup.GetTabByTabId(tabid);
            if (tab == null)
                return false;
            var dom = tab.Doms[docid];
            if (dom == null)
                return false;

            return true;
        }
        catch (err) {
            gHeadup.LogError('IsEnabled', err);
            return false;
        }
    }

    HeadupClass.prototype.CaptureMouse = function(tabid, docid) {
        try {
            this.ReleaseCaptureMouse(tabid, docid);

            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            dom.addEventListener("mousemove", this.OnMouseMove, true, true);
        }
        catch (err) {
            gHeadup.LogError('CaptureMouse', err);
        }
    }

    HeadupClass.prototype.OnMouseMove = function(event) {
        try {
            var dom = event.target.ownerDocument;
            dom.mouseX = event.pageX;
            dom.mouseY = event.pageY;
        }
        catch (err) {
            gHeadup.LogError('OnMouseMove', err);
        }
    }

    HeadupClass.prototype.ReleaseCaptureMouse = function(tabid, docid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            dom.removeEventListener("mousemove", this.OnMouseMove, true);
            dom.mouseX = 0;
            dom.mouseY = 0;
        }
        catch (err) {
            gHeadup.LogError('ReleaseCaptureMouse', err);
        }
    }

    HeadupClass.prototype.GetMousePosition = function(tabid, docid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var result = {};
            result['mousex'] = dom.mouseX;
            result['mousey'] = dom.mouseY;
            return result;
        }
        catch (err) {
            gHeadup.LogError('GetMousePosition', err);
        }
    }
}
catch (err) {
    LogError('error prototyping XulHtmlDocument: ' + err);
}
