
try {

    HeadupClass.prototype.CharmNextGuid = 0;

    HeadupClass.prototype.GetCharmNextGuid = function() {
        this.CharmNextGuid++;
        return this.CharmNextGuid.toString();
    }

    HeadupClass.prototype.TabFromParams = function(params) {
        try {
            var tabid = params['tabid'];
            return gHeadup.GetTabByTabId(tabid);
        }
        catch (err) {
            gHeadup.LogError('TabFromParams', err);
        }
    }

    HeadupClass.prototype.GetElementFromParams = function(params) {
        try {
            var doc = GetDomFromParams(params);
            var guid = params['guid'];
            return doc.CharmGuids[guid];
        }
        catch (err) {
            gHeadup.LogError('GetElementFromParams', err);
        }
    }

    HeadupClass.prototype.GetDomFromParams = function(params) {
        try {
            var tab = this.TabFromParams(params);
            var docid = params['docid'];
            return tab.Doms[docid];
        }
        catch (err) {
            gHeadup.LogError('GetDomFromParams', err);
        }
    }

    HeadupClass.prototype.GetElementGuid = function(element, dom) {
        try {
            if (element.CharmGuid != null) {
                return element.CharmGuid;
            }

            var guid = this.GetCharmNextGuid();
            dom.CharmGuids[guid] = element;
            element.CharmGuid = guid;
            return guid;
        }
        catch (err) {
            gHeadup.LogError('GetElementGuid', err);
        }
    }

    HeadupClass.prototype.GetProperty = function(tabid, docid, elid, name, convertToGuid) {
        try {
            var tab = gHeadup.GetTabByTabId(tabid);
            var dom = tab.Doms[docid];
            var element = dom.CharmGuids[elid];
            var res = element[name];
            if (res == null)
                return null;
            if (convertToGuid) {
                if (res.tagName.toLowerCase() == 'html')
                    return null;
                return this.GetElementGuid(res, dom);
            }
            else
                return res;
        }
        catch (err) {
            gHeadup.LogError('GetProperty', err);
            return null;
        }
    }

    HeadupClass.prototype.GetProperties = function(tabid, docid, elid, names) {
        try {
            var tab = gHeadup.GetTabByTabId(tabid);
            var dom = tab.Doms[docid];
            var element = dom.CharmGuids[elid];
            return gHeadup.GetPropertiesFromElement(element, names);
        }
        catch (err) {
            gHeadup.LogError('GetProperties', err);
            return null;
        }
    }

    HeadupClass.prototype.GetPropertiesFromElement = function(element, names) {
        try {
            var res = new Array(names.length);
            for (var i = 0; i < names.length; i++) {
                res[i] = element[names[i]];
            }
            return res;
        }
        catch (err) {
            gHeadup.LogError('GetPropertiesFromElement', err);
            return null;
        }
    }

    HeadupClass.prototype.GetAttribute = function(tabid, docid, elid, name) {
        try {
            return gHeadup.GetTabByTabId(tabid).Doms[docid].CharmGuids[elid].getAttribute(name);
        }
        catch (err) {
            gHeadup.LogError('GetAttribute', err);
            return null;
        }
    }

    HeadupClass.prototype.GetParent = function(tabid, docid, elid) {
        try {
            var tab = gHeadup.GetTabByTabId(tabid);
            var dom = tab.Doms[docid];
            var element = dom.CharmGuids[elid];
            var parentElement = element.parentNode;
            if (parentElement == null) {
                return null;
            }
            else {
                var guid = this.GetElementGuid(parentElement, dom);
                return guid;
            }
        }
        catch (err) {
            gHeadup.LogError('GetParent', err);
            return null;
        }
    }

    HeadupClass.prototype.GetChildren = function(tabid, docid, elid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var element = dom.CharmGuids[elid];
            var data = new Array(element.childNodes.length);

            for (var i = 0; i < element.childNodes.length; i++) {
                var child = element.childNodes[i];
                var guid = this.GetElementGuid(child, dom);
                data[i] = guid;
            }
            return data;
        }
        catch (err) {
            gHeadup.LogError('GetChildren', err);
            return null;
        }
    }

    HeadupClass.prototype.SetStyleAttribute = function(tabid, docid, elid, name, value) {
        try {
            var element = gHeadup.GetTabByTabId(tabid).Doms[docid].CharmGuids[elid];
            element.style[name] = value;
        }
        catch (err) {
            gHeadup.LogError('SetStyleAttribute', err);
        }
    }

    HeadupClass.prototype.AppendChild = function(tabid, docid, elid, elementGuid, referenceGuid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var element = dom.CharmGuids[elementGuid];
            var parent = dom.CharmGuids[elid];

            parent.appendChild(element);
        }
        catch (err) {
            gHeadup.LogError('AppendChild', err);
        }
    }

    HeadupClass.prototype.InsertBefore = function(tabid, docid, elid, elementGuid, referenceGuid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var element = dom.CharmGuids[elementGuid];
            var reference = dom.CharmGuids[referenceGuid];
            var parent = dom.CharmGuids[elid];

            parent.insertBefore(element, reference);
        }
        catch (err) {
            gHeadup.LogError('AppendChild', err);
        }
    }

    HeadupClass.prototype.CreateAndAppend = function(tabid, docid, elid, tagName, textContent, referenceGuid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];

            var reference = dom.CharmGuids[referenceGuid];
            var parent = dom.CharmGuids[elid];

            var element = dom.createElement(tagName);
            var guid = gHeadup.GetElementGuid(element, dom);
            element.textContent = textContent;
            parent.insertBefore(element, reference);
            return guid;
        }
        catch (err) {
            gHeadup.LogError('CreateAndAppend', err);
            return null;
        }
    }

    HeadupClass.prototype.SurroundContents = function(tabid, docid, elid, tagName, startIndex, endIndex) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];

            var parent = dom.CharmGuids[elid];
            var element = dom.createElement(tagName);
            var guid = gHeadup.GetElementGuid(element, dom);

            var docRange = document.createRange();
            docRange.selectNode(parent);
            docRange.setStart(parent, startIndex);
            docRange.setEnd(parent, endIndex);
            docRange.surroundContents(element);
            return guid;
        }
        catch (err) {
            gHeadup.LogError('SurroundContents', err);
        }
    }

    HeadupClass.prototype.ReplaceChild = function(tabid, docid, elid, elementGuid, oldGuid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var element = dom.CharmGuids[elementGuid];
            var reference = dom.CharmGuids[oldGuid];
            var parent = dom.CharmGuids[elid];
            parent.replaceChild(element, reference);
        }
        catch (err) {
            gHeadup.LogError('ReplaceChild', err);
        }
    }


    HeadupClass.prototype.RemoveChild = function(tabid, docid, elid, elementGuid) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var element = dom.CharmGuids[elementGuid];
            var parent = dom.CharmGuids[elid];
            parent.removeChild(element);
        }
        catch (err) {
            gHeadup.LogError('RemoveChild', err);
        }
    }

    HeadupClass.prototype.SetProperty = function(tabid, docid, elid, name, value) {
        try {
            var element = gHeadup.GetTabByTabId(tabid).Doms[docid].CharmGuids[elid];
            element[name] = value;
        }
        catch (err) {
            gHeadup.LogError('SetProperty', err);
        }
    }

    HeadupClass.prototype.AttachEvent = function(tabid, docid, elid, name) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            var element = dom.CharmGuids[elid];
            var fixedName = name;
            if (fixedName == "onclick")
                fixedName = "click";

            var func = function(event) {
                var data = {};
                data['name'] = name;
                data['this'] = elid;
                gHeadup.OnEventOnPage(event, dom, data);
            };

            element.addEventListener(fixedName, func, true, true);
            if (element['AttachedFunctions'] == null)
                element['AttachedFunctions'] = {};
            element['AttachedFunctions'][fixedName] = func;
        }
        catch (err) {
            gHeadup.LogError('AttachEvent', err);
        }
    }

    HeadupClass.prototype.DetachEvent = function(tabid, docid, elid, name) {
        try {
            var dom = gHeadup.GetTabByTabId(tabid).Doms[docid];
            if (dom == null)
                return;
            var element = dom.CharmGuids[elid];
            var fixedName = name;
            if (fixedName == "onclick")
                fixedName = "click";

            var func = element['AttachedFunctions'][fixedName];
            element.removeEventListener(fixedName, func, true);
        }
        catch (err) {
            gHeadup.LogError('DetachEvent', err);
        }
    }

    HeadupClass.prototype.OnEventOnPage = function(event, dom, data) {
        try {
            data['target'] = this.GetElementGuid(event.target, dom);
            data['newValue'] = event['newValue'];
            data['prevValue'] = event['prevValue'];
            data['mouseX'] = event['pageX'];
            data['mouseY'] = event['pageY'];

            gHeadup.FireEventInSingleWindow("EventOnPage", data);
        }
        catch (err) {
            gHeadup.LogError('OnEventOnPage', e);
        }
    }

    HeadupClass.prototype.PathToGuid = function(path, page) {
        try {
            var targetElement = this.GetElementByIndexPath(path, page);
            var targetGuid = this.GetElementGuid(targetElement, page);
            return targetGuid;
        }
        catch (err) {
            gHeadup.LogError('PathToGuid', err);
        }
    }

    HeadupClass.prototype.GetElementByIndexPath = function(path, root) {
        var reduce = function ReduceGetElement(root, path) {
            var pathSteps = path.split("-");

            if (pathSteps.length > 0 && pathSteps[0] != "") {
                var index = parseInt(pathSteps[0]);
                pathSteps.splice(0, 1);
                return reduce(root.childNodes[index], pathSteps.join("-"));
            }

            return root;
        };

        return reduce(root, path);
    }

    HeadupClass.prototype.CreatePathByNodeIndex = function(node, root) {

        var reduce = function(node, path) {
            var parent = node.parentNode;
            if (parent == null || node == root) {
                return path;
            }

            var children = parent.childNodes;

            for (var i = 0; i < children.length; i++) {
                var child = children[i];
                if (node == child) {
                    var index = i;
                    var newPath = path == "" ? index : index + "-" + path;
                    return reduce(parent, newPath);
                }
            }
        };

        return reduce(node, "");
    }

    HeadupClass.prototype.GetAttributes = function(tabid, docid, elid) {
        try {
            var element = gHeadup.GetTabByTabId(tabid).Doms[docid].CharmGuids[elid];
            var attrs = element.attributes;
            var data = new Array(attrs.length * 2);
            for (var i = 0; i < attrs.length; i++) {
                data[i * 2] = attrs[i].name;
                data[(i * 2) + 1] = attrs[i].value;
            }
            return data;
        }
        catch (err) {
            gHeadup.LogError('GetAttributes', err);
            return null;
        }
    }

    HeadupClass.prototype.GetDataFromEvent = function(event) {
        var id = event.detail;
        var params = {};
        var dataElement = event.target.getElementById("methoddata" + id);
        if (dataElement == null) {
            gHeadup.LogError('GetParamsFromEvent', 'no dataElement');
            return params;
        }

        for (var i = 0; i < dataElement.childNodes.length; i++) {
            var key = dataElement.childNodes[i].id;
            var value = dataElement.childNodes[i].childNodes[0].id;
            params[key] = value;
        }
        dataElement.parentNode.removeChild(dataElement);
        return params;
    }

}
catch (err) {
    gHeadup.LogError('error prototyping XulHtmlElement: ' + err);
}